#ifndef BOARD
#define BOARD

#include <map>
#include <string>

#include "Piece.hpp"
#include "Coord.hpp"
#include "RookingPosibility.hpp"
#include "EnpassantPosibility.hpp"
#include "Platform.hpp"

class Board {
public:
    Board();
    static Board from_fen(std::string);
    static Board initial();
    static Board empty();

    std::map<Coord, Piece> squares;
    Color turn_color;
    RookingPosibility rooking_posibility;
    EnpassantPosibility enpassant_posibility;
    UNumber unused_moves_count;
    UNumber moves_count;

	void add_piece(const Piece&, const Coord&);
	void remove_piece(const Coord&);
	void move_piece(const Coord&, const Coord&);

	bool has_piece_at(const Coord&) const;
	Piece get_piece_at(const Coord&) const;

	bool line_is_empty(const Coord&, const Coord&) const;

    bool is_empty() const;
    UNumber pieces_count() const;
    std::string to_fen() const;

    bool operator==(const Board&) const;
    bool operator!=(const Board&) const;
private:
};

#endif
