#include "Coord.hpp"
#include "Exceptions.hpp"

Coord::Coord()
    : Coord(1,1)
{
    // none
}

Coord::Coord(UNumber file, UNumber rank)
    : file(file), rank(rank)
{
    if (Coord::is_valid(file, rank)) return;

    throw CustomException("Invalid Coords in constructor");
}

Coord Coord::from_fen(std::string fen) {
    const UNumber file = fen.at(0) - 'a' + 1;
    const UNumber rank = fen.at(1) - '0';

    return Coord(file, rank);
}

bool Coord::is_valid(UNumber file, UNumber rank) {
    return (1 <= file && file <= 8) && (1 <= rank && rank <= 8);
}

std::string Coord::to_fen() const {
    std::string result = "";

    result += (*this).file - 1 + 'a';
    result += (*this).rank + '0';

    return result;
}

bool Coord::operator==(const Coord& rhs) const {
    return (*this).file == rhs.file && (*this).rank == rhs.rank;
}

bool Coord::operator!=(const Coord& rhs) const {
    return (*this).file != rhs.file || (*this).rank != rhs.rank;
}

bool Coord::operator<(const Coord& rhs) const {
    if ((*this).rank != rhs.rank) return (*this).rank < rhs.rank;

    return (*this).file < rhs.file;
}
