#ifndef MOVE
#define MOVE

#include <string>

#include "Coord.hpp"
#include "Piece.hpp"
#include "Board.hpp"
#include "RestoreData.hpp"

class Move {
public:
	static Move none();
	static Move silent(const Coord&, const Coord&);
	static Move pawn_double(const Coord&, const Coord&);
    static Move capture(const Coord&, const Coord&);
    static Move enpassant(const Coord&, const Coord&);
    static Move rooking(const Coord&, const Coord&);
    static Move promotion(const Coord&, const Coord&, const PieceType&);
    static Move from_fen(std::string, const Board&);

    Coord from, to;
    PieceType promoted;

    bool is_silent() const;
    bool is_capture() const;
    bool is_enpassant() const;
    bool is_promotion() const;
    bool is_rooking() const;
    std::string to_fen() const;

    RestoreData make(Board&) const;

    bool operator==(const Move&) const;
    bool operator!=(const Move&) const;
private:
    enum class MoveType {NONE, SILENT, CAPTURE, ENPASSANT, ROOKING, PROMOTION, PAWN_DOUBLE};
    Move(const MoveType&, const Coord&, const Coord&, const PieceType&);

    MoveType type;
};

#endif
